#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <byteswap.h>
#define _BSD_SOURCE 1
#include <endian.h>

#define P64(_u) (uint32_t)((_u) >> 32), (uint32_t)(_u)

int main (int argc,
          char* argv[])
{
  uint8_t b8[] = { 0x12, 0x34, 0x56, 0x78,
                   0x9a, 0xbc, 0xde, 0xf0 };
  union {
    uint8_t u8[2];
    uint16_t u16;
  } hbo16;
  uint16_t u16, l16, b16;
  union {
    uint8_t u8[4];
    uint32_t u32;
  } hbo32;
  uint32_t u32, l32, b32;
  union {
    uint8_t u8[8];
    uint32_t u32[2];
    uint64_t u64;
  } hbo64;
  uint64_t u64, l64, b64;

  memcpy(hbo16.u8, b8, sizeof(hbo16));
  memcpy(hbo32.u8, b8, sizeof(hbo32));
  memcpy(hbo64.u8, b8, sizeof(hbo64));
  u16 = bswap_16(hbo16.u16);
  printf("HBO16 %04x -> %04x\n", hbo16.u16, u16);
  u32 = bswap_32(hbo32.u32);
  printf("HBO32 %08lx -> %08lx\n", hbo32.u32, u32);
  u64 = bswap_64(hbo64.u64);
  printf("HBO64 %08lx%08lx -> %08lx%08lx\n", P64(hbo64.u64), P64(u64));

  l16 = htole16(u16);
  b16 = htobe16(u16);
  printf("H %04x LE %04x BE %04x rLE %04x rBE %04x\n",
         u16, l16, b16, le16toh(l16), be16toh(b16));

  l32 = htole32(u32);
  b32 = htobe32(u32);
  printf("H %08lx LE %08lx BE %08lx rLE %08lx rBE %08lx\n",
         u32, l32, b32, le32toh(l32), be32toh(b32));

  l64 = htole64(u64);
  b64 = htobe64(u64);
  printf("H %08lx%08lx\n", P64(u64));
  printf("LE %08lx%08lx BE %08lx%08lx\n", P64(l64), P64(b64));
  {
    uint64_t rl64 = le64toh(l64);
    uint64_t rb64 = be64toh(b64);
    printf("rLE %08lx%08lx rBE %08lx%08lx\n", P64(rl64), P64(rb64));
  }
}
